/*****************************************************************************/
/*									     */
/*									     */
/*	Xpack version 1.00 -- module tools.c				     */
/*									     */
/*	Miscellaneous utility functions.				     */
/*	Written by Michael Bischoff (mbi@mo.math.nat.tu-bs.de)		     */
/*	November-1994							     */
/*	see COPYRIGHT.xpack for Copyright details			     */
/*									     */
/*									     */
/*****************************************************************************/
#ifndef _POSIX_SOURCE
#define _POSIX_SOURCE
#endif
#include "xpack.h"

int ilog2(long n) {        	/* returns the number of bits in n */
    register int log;

    if (n < 0)
	fatal("ilog2(%ld): negative argument", n);
    for (log = 0; n; ++log)	/* =>  n < 2^bits		     */
	n >>= 1;
    return log;
}
int isqrt(long n) {	/* returns floor(sqrt(n)) */
    int intsqrt, pos;
    if (n < 0)
	fatal("isqrt(%ld): negative argument", n);
    if (!n)
	return 0;
    intsqrt = 1 << (pos = (ilog2(n) - 1) >> 1);
    /* now intsqrt is the maximal power of 2 not greater than sqrt(n) */
    n -= 1L << (pos << 1);
    /* n is the remainder of n - intsqrt^2 */
    while (pos) {
	long test;
	--pos;
	test = (1 << (pos<<1)) + (intsqrt << (pos+1));
	if (n >= test) {
	    /* yes, bit pos may be set */
	    n -= test;
	    intsqrt |= 1 << pos;
	}
    }
    return intsqrt;
}


void fatal(const char *msg, ...) {
    va_list args;

    va_start(args, msg);
    vfprintf(stderr, msg, args);
    fprintf (stderr, "\n");
    exit (1);
}

void *malloc_(size_t n) {
    void *p;
    if (!n)
	return NULL;	/* since malloc(0) may return NULL */
    p = malloc(n);
    if (!p)
	fatal("out of memory");
    return p;
}

void *calloc_(size_t n, size_t s) {
    void *p;
    if (!n)
	return NULL;	/* WATCOM C says "out of memory" in the case n = 0 */
    if (!(p = calloc(n, s)))
        fatal("out of memory");
    return p;
}

void *realloc_(void *p, size_t n) {
    if (p == NULL)	/* no old block of size > 0 exists */
	return malloc_(n);
    if (!n) {
	free_(p);
	return NULL;
    }
    if (!(p = realloc(p, n)))
        fatal("out of memory\n");
    return p;
}

void free_(void *p) {
    if (p)
	free(p);
}

char *strsav(const char *txt) {
    char *p = malloc_(1 + strlen(txt));
    strcpy(p, txt);
    return p;
}
